/*
 * MP3DecoderDriver.cpp
 *
 *  Created on: Apr 23, 2018
 *      Author: Ameer
 */

#include "MP3DecoderDriver.h"
#include <stdio.h>

uint8_t SCI_MODE = 0x00;
uint8_t SCI_STATUS = 0x01;
uint8_t SCI_BASS = 0x02;
uint8_t SCI_CLOCKF = 0x03;
uint8_t SCI_DECODE_TIME = 0x04;
uint8_t SCI_AUDATA = 0x05;
uint8_t SCI_WRAM = 0x06;
uint8_t SCI_WRAMADDR = 0x07;
uint8_t SCI_HDAT0 = 0x08;
uint8_t SCI_HDAT1 = 0x09;
uint8_t SCI_AIADDR = 0x0A;
uint8_t SCI_VOL = 0x0B;
uint8_t SCI_AICTRL0 = 0x0C;
uint8_t SCI_AICTRL1 = 0x0D;
uint8_t SCI_AICTRL2 = 0x0E;
uint8_t SCI_AICTRL3 = 0x0F;

void mp3_decoder_init()
{
    ssp__init(24);
    gpio_x__set_as_output(1, 28); //Set pin 1.28 as output for RESET signal
    gpio_x__set_as_output(1, 20); //Set pin 1.20 as output for DCS signal
    gpio_x__set_as_output(1, 31); //Set pin 1.31 as output for CS signal

    //gpio_x__set(0, 25, true); //Set pin 0.25 high initially
    //gpio_x__set(0, 26, true); //Set pin 0.26 high initially
    //gpio_x__set(1, 30, true); //Set pin 1.30 high initially
    gpio_x__set_high(1, 28);
    gpio_x__set_high(1, 20);
    gpio_x__set_high(1, 31);

    gpio_x__set_as_input(0, 26); //Set pin 0.26 as input for DREQ

    write_to_decoder(SCI_MODE, 0x0800);         //Set up MODE register
    //write_to_decoder();                         //Set up Bass/Treble register
    write_to_decoder(SCI_CLOCKF, 0x2000);       //Set up Clock register
    write_to_decoder(SCI_AUDATA, 0xAC45);       //Set up AudioData register               `
    write_to_decoder(SCI_VOL, 0x2424);          //Set up Volume register

    uint16_t data1 = read_from_decoder(SCI_MODE);
    uint16_t data2 = read_from_decoder(SCI_CLOCKF);
    uint16_t data3 = read_from_decoder(SCI_AUDATA);
    uint16_t data4 = read_from_decoder(SCI_VOL);
    printf("SCI_MODE = %x\n", data1);
    printf("SCI_CLOCKF = %x\n", data2);
    printf("SCI_AUDATA = %x\n", data3);
    printf("SCI_VOL = %x\n", data4);

    //u0_dbg_printf("mode data = %x\n", data);

}

void write_to_decoder(uint8_t addr, uint16_t data_out)
{
    while(!(gpio_x__get_level(0, 26))) //Check DREQ pin
    {
        //vTaskDelay(1);
    }

    gpio_x__set(1, 31, false);
    //gpio_x__set_low(1, 30);
    //gpio_x__set_low(0, 26);

    ssp__exchange_byte(0x02);
    ssp__exchange_byte(addr);
    
    uint8_t highByte = (data_out >> 8) & 0xFF;
    //printf("write highByte = %x\n", highByte);
    ssp__exchange_byte(highByte);

    uint8_t lowByte = data_out & 0xFF;
    //printf("write lowByte = %x\n", lowByte);
    ssp__exchange_byte(lowByte);

    while(!(gpio_x__get_level(0, 26)))
    {
        //vTaskDelay(1);
    }

    gpio_x__set(1, 31, true);
    //gpio_x__set_high(1, 30);
    //gpio_x__set_high(0, 26);
}

uint16_t read_from_decoder(uint8_t addr)
{
    while(!(gpio_x__get_level(0, 26)))
    {
        //vTaskDelay(1);
    }

    gpio_x__set(1, 31, false);
    //gpio_x__set_low(1, 30);
    //gpio_x__set_low(0, 26);

    ssp__exchange_byte(0x03);
    ssp__exchange_byte(addr);
    uint8_t highByte = ssp__exchange_byte(0xFF);
    //printf("read highByte = %x\n", highByte);

    while(!(gpio_x__get_level(0, 26)))
    {
        //vTaskDelay(1);
    }

    uint8_t lowByte = ssp__exchange_byte(0xFF);
    //printf("read lowByte = %x\n", lowByte);

    while(!(gpio_x__get_level(0, 26)))
    {
        //vTaskDelay(1);
    }

    gpio_x__set(1, 31, true);
    //gpio_x__set_high(1, 30);
    //gpio_x__set_high(0, 26);

    uint16_t dataread = highByte << 8;
    dataread |= lowByte;

    return dataread;
}

void play_mp3_track(uint8_t buffer[])
{
    for(uint8_t byteIndex = 0; byteIndex < 16; byteIndex++)
    {
        while(!(gpio_x__get_level(0, 26)))
        {
            //vTaskDelay(1);
        }

        gpio_x__set(1, 20, false);
        //gpio_x__set_low(0, 26);
        //gpio_x__set_low(1, 30);

        for(uint16_t i = 32 * byteIndex; i < (32 * byteIndex) + 32; i++)
        {
            ssp__exchange_byte(buffer[i]);
        }

        while(!(gpio_x__get_level(0, 26)))
        {
            //vTaskDelay(1);
        }

        gpio_x__set(1, 20, true);
        //gpio_x__set_high(0, 26);
        //gpio_x__set_high(1, 30);
    }
}

/*
#include "MP3DecoderDriver.hpp"
#include "printf_lib.h"
#include "utilities.h"
#include "tasks.hpp"

#define SCI_MODE 0x00
#define SCI_STATUS 0x01
#define SCI_BASS 0x02
#define SCI_CLOCKF 0x03
#define SCI_DECODE_TIME 0x04
#define SCI_AUDATA 0x05
#define SCI_WRAM 0x06
#define SCI_WRAMADDR 0x07
#define SCI_HDAT0 0x08
#define SCI_HDAT1 0x09
#define SCI_AIADDR 0x0A
#define SCI_VOL 0x0B
#define SCI_AICTRL0 0x0C
#define SCI_AICTRL1 0x0D
#define SCI_AICTRL2 0x0E
#define SCI_AICTRL3 0x0F

uint8_t volume_level = 5;

void MP3Decoder::init()
{
    audio.init(audio.SSP0,0x08,audio.SPI,8);    //Have SPP0 take care of communications between decoder

    LPC_GPIO1->FIODIR |= (1<<19);   //set pin P1.19 as output for RESET signal
    LPC_PINCON->PINSEL3 &= ~(3<<6);

    LPC_GPIO1->FIODIR |= (1<<20);   //set pin P1.20 as output for DCS signal
    LPC_PINCON->PINSEL3 &= ~(3<<8);

    LPC_GPIO1->FIODIR |= (1<<22);   //set pin P1.22 as output for CS signal
    LPC_PINCON->PINSEL3 &= ~(3<<12);

    LPC_GPIO1->FIOSET = (1<<19);    //set pin P1.19 high initially
    LPC_GPIO1->FIOSET = (1<<20);    //set pin P1.20 high initially
    LPC_GPIO1->FIOSET = (1<<22);    //set pin P1.22 high initially

    LPC_GPIO1->FIODIR &= ~(1<<23);  //set pin P1.23 as input for DREQ
    LPC_PINCON->PINSEL3 &= ~(3<<14);

    write_to_decoder(SCI_MODE, 0x0800);         //Set up MODE register
    //write_to_decoder();                         //Set up Bass/Treble register
    write_to_decoder(SCI_CLOCKF, 0x2000);       //Set up Clock register
    write_to_decoder(SCI_AUDATA, 0xAC45);       //Set up AudioData register               `
    //write_to_decoder(SCI_VOL, 0x2424);          //Set up Volume register
    volumeControl(true, true);
    //uint16_t data = read_from_decoder(SCI_VOL);
    //u0_dbg_printf("mode data = %x\n", data);

}
void MP3Decoder::write_to_decoder(uint8_t addr, uint16_t data_out)
{
    while(!(LPC_GPIO1->FIOPIN & (1 << 23)))
    {
        //vTaskDelay(1);
    }

    LPC_GPIO1->FIOCLR = (1 << 22);
    audio.transfer(0x02);
    audio.transfer(addr);

    uint8_t highByte = (data_out>>8) & 0xFF;
    //u0_dbg_printf("highByte = %x\n", highByte);
    audio.transfer(highByte);

    uint8_t lowByte = data_out & 0xFF;
    //u0_dbg_printf("lowByte = %x\n", lowByte);
    audio.transfer(lowByte);

    while(!(LPC_GPIO1->FIOPIN & (1 << 23)))
    {
        //vTaskDelay(1);
    }

    LPC_GPIO1->FIOSET = (1 << 22);
}

uint16_t MP3Decoder::read_from_decoder(uint8_t addr)
{
    while(!(LPC_GPIO1->FIOPIN & (1 << 23)));
    {
        //vTaskDelay(1);
    }

    LPC_GPIO1->FIOCLR = (1 << 22);
    audio.transfer(0x03);
    audio.transfer(addr);
    uint8_t highByte = audio.transfer(0xFF);

    while(!(LPC_GPIO1->FIOPIN & (1 << 23)))
    {
        //vTaskDelay(1);
    }

    uint8_t lowByte = audio.transfer(0xFF);

    while(!(LPC_GPIO1->FIOPIN & (1 << 23)))
    {
        //vTaskDelay(1);
    }

    LPC_GPIO1->FIOSET = (1 << 22);

    uint16_t dataRead = highByte << 8;
    dataRead |= lowByte;
    //u0_dbg_printf("dataRead: %x\n", dataRead);

    return dataRead;
}

void MP3Decoder::playMP3Track(uint8_t buffer[])
{
    for(uint8_t byteIndex = 0; byteIndex < 16; byteIndex++)         //Iterate through all 16 32 byte chunks
    {
        //u0_dbg_printf("byteIndex = %d\n", byteIndex);
        while(!(LPC_GPIO1->FIOPIN & (1 << 23)))                     //Wait until DREQ is set to high
        {
            vTaskDelay(1);
        }

        LPC_GPIO1->FIOCLR = (1 << 20);                              //Select SDI CS
        for(uint16_t i = 32 * byteIndex; i < (32 * byteIndex) + 32; i++)
        {
            //u0_dbg_printf("i = %d, audio transferring.\n", i);
            MP3Decoder::audio.transfer(buffer[i]);           //Send next 32 bytes to decoder
            //delay_ms(100);
        }

        while(!(LPC_GPIO1->FIOPIN & (1 << 23)))                      //Wait until DREQ is set to high
        {
            vTaskDelay(1);
        }

        LPC_GPIO1->FIOSET = (1 << 20);                              //Deselect SDI CS
    }
}

void MP3Decoder::volumeControl(bool higher, bool init)
{
    if(higher && volume_level < 8 && !init)
    {
        volume_level++;
    }
    else if(!higher && volume_level > 1 && !init){
        volume_level--;
    }

    if(volume_level == 8)
    {
        write_to_decoder(SCI_VOL, 0x1010);
        u0_dbg_printf("volume level = %i", volume_level);
    }
    else if(volume_level == 7)
    {
        write_to_decoder(SCI_VOL, 0x2020);
        u0_dbg_printf("volume level = %i", volume_level);
    }
    else if(volume_level == 6)
    {
        write_to_decoder(SCI_VOL, 0x2525);
        u0_dbg_printf("volume level = %i", volume_level);
    }
    else if(volume_level == 5)
    {
        write_to_decoder(SCI_VOL, 0x3030);
        u0_dbg_printf("volume level = %i", volume_level);
    }
    else if(volume_level == 4)
    {
        write_to_decoder(SCI_VOL, 0x3535);
        u0_dbg_printf("volume level = %i", volume_level);
    }
    else if(volume_level == 3)
    {
        write_to_decoder(SCI_VOL, 0x4040);
        u0_dbg_printf("volume level = %i", volume_level);
    }
    else if(volume_level == 2)
    {
        write_to_decoder(SCI_VOL, 0x4545);
        u0_dbg_printf("volume level = %i", volume_level);
    }
    else if(volume_level == 1)
    {
        write_to_decoder(SCI_VOL, 0xFEFE);
        u0_dbg_printf("volume level = %i", volume_level);
    }
    vTaskDelay(1000);
}
*/
